<?php
namespace Modules\Cadastros\Controllers;

use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Message;

class Sup01600CalculoDaQuantidadeMedia{

    protected $connection;

    public function __construct ($connection){
    	$this->connection = $connection;
    }

	public function calcular($request, $response) {
		try {
			$row = (array)$request->getRow();
			$this->connection->beginTransaction();

			$this->zeraMediaEstoqueCompra($row);
			$this->insereProdutoSolicitacao($row);
			$this->calculoEstoque($row);

			$this->connection->commit();
		} catch (\Exception $e) {
			$response->setError(new Error($e->getMessage(), 500));
			$this->connection->rollback();
		}
	}
	private function zeraMediaEstoqueCompra($row){
		$sql = "UPDATE PRODPREFMED
				SET QTMEDSOLPROD = NULL
				WHERE (INSTR(:CDFILIAL, PRODPREFMED.CDFILIAL) <> 0 OR :CDFILIAL = 'T')
				AND CDPRODUTO IN (SELECT CDPRODUTO
				FROM PRODUTO
				WHERE CDARVPROD BETWEEN :CDPRODINI AND :CDPRODFIN)";
		$params = array(
			'CDFILIAL' => $row['CDFILIAL'],
			'CDPRODINI' => $row['CDPRODINI'],
			'CDPRODFIN' => str_pad($row['CDPRODFIN'], 20, "z", STR_PAD_LEFT)
		);
		$this->execute($sql, $params);
	}
	private function insereProdutoSolicitacao($row){
		$sql = "INSERT INTO PRODPREFMED(CDFILIAL, CDPRODUTO)
				SELECT DISTINCT ITEMSOLI.CDFILSOLI, PRODUTO.CDPRODESTO
				 FROM ITEMSOLI,PRODUTO
				 WHERE ((INSTR(:CDFILIAL,ITEMSOLI.CDFILSOLI) <> 0) OR (:CDFILIAL = 'T'))
				 AND (ITEMSOLI.DTPROGENTR BETWEEN :DTINICIAL AND :DTFINAL)
				 AND (PRODUTO.CDARVPROD BETWEEN :CDPRODINI AND :CDPRODFIN)
				 AND ITEMSOLI.CDPRODUTO = PRODUTO.CDPRODUTO
				 AND ITEMSOLI.CDPRODUTO = PRODUTO.CDPRODESTO
				 AND NOT EXISTS(SELECT PRODPREFMED.CDFILIAL, PRODPREFMED.CDPRODUTO
				 FROM PRODPREFMED
				 WHERE PRODPREFMED.CDFILIAL = ITEMSOLI.CDFILSOLI
				 AND PRODPREFMED.CDPRODUTO = PRODUTO.CDPRODESTO)";
		$params = array(
			'CDFILIAL' => $row['CDFILIAL'],
			'CDPRODINI' => $row['CDPRODINI'],
			'CDPRODFIN' => str_pad($row['CDPRODFIN'], 20, "z", STR_PAD_LEFT),
			'DTINICIAL' => $row['INTERVALO_DATA']['start'],
			'DTFINAL' => $row['INTERVALO_DATA']['end']
		);
		$this->execute($sql, $params);
	}

	private function calculoEstoque($row){
		$sql = "UPDATE (SELECT CDFILIAL, CDPRODUTO, QTMEDSOLPROD
				FROM PRODPREFMED
				WHERE (CDFILIAL, CDPRODUTO) IN (SELECT DISTINCT ITEMSOLI.CDFILSOLI,
				PRODUTO.CDPRODUTO
				FROM PRODUTO, ITEMSOLI
				WHERE ITEMSOLI.CDPRODUTO = PRODUTO.CDPRODESTO
				AND ITEMSOLI.CDPRODUTO = PRODUTO.CDPRODUTO
				AND (INSTR(:CDFILIAL,ITEMSOLI.CDFILSOLI) <> 0) OR (:CDFILIAL = 'T')
				AND ITEMSOLI.DTPROGENTR BETWEEN :DTINICIAL AND :DTFINAL
				AND PRODUTO.CDARVPROD BETWEEN :CDPRODINI AND :CDPRODFIN)) A
				SET QTMEDSOLPROD = (SELECT AVG(ITEMSOLI.QTSOLIPROD*PRODUTO.VRFATOCONV)
				FROM PRODUTO, ITEMSOLI
				WHERE ITEMSOLI.CDFILSOLI = A.CDFILIAL
				AND PRODUTO.CDPRODUTO = A.CDPRODUTO
				AND PRODUTO.CDPRODUTO = ITEMSOLI.CDPRODUTO
				AND ITEMSOLI.DTPROGENTR BETWEEN :DTINICIAL AND :DTFINAL)";
		$params = array(
			'CDFILIAL' => $row['CDFILIAL'],
			'CDPRODINI' => $row['CDPRODINI'],
			'CDPRODFIN' => str_pad($row['CDPRODFIN'], 20, "z", STR_PAD_LEFT),
			'DTINICIAL' => $row['INTERVALO_DATA']['start'],
			'DTFINAL' => $row['INTERVALO_DATA']['end']
		);
		$this->execute($sql, $params);
	}

	private function execute($sql, $params){
		$query = $this->connection->prepare($sql);
		$query->execute($params);
	}
}